@@ -6,7 +6,7 @@ import java.text.*;
 
 import javax.swing.*;
 import javax.swing.JSpinner.*;
-import javax.swing.text.*;
+import javax.swing.event.*;
 
 import net.argius.stew.*;
 
@@ -21,6 +21,8 @@ final class Menu extends JMenuBar {
     final JCheckBoxMenuItem checkBoxStatusBarVisibility;
     final JCheckBoxMenuItem checkBoxShowColumnNumber;
 
+    Object focused;
+
     private final Environment env;
     private final WindowOutputProcessor op;
     private final Window window;
@@ -36,102 +38,113 @@ final class Menu extends JMenuBar {
      * @param env s
      * @param op WindowOutputProcessor
      */
-    Menu(Environment env, WindowOutputProcessor op) {
+    Menu(Environment env, final WindowOutputProcessor op) {
         this.env = env;
         this.op = op;
         this.window = op.window;
         this.consoleTextArea = op.consoleTextArea;
         this.resultSetTable = op.resultSetTable;
         this.statusBarLabel = op.statusBarLabel;
+        FocusListener focusListener = new FocusAdapter() {
+
+            public void focusGained(FocusEvent e) {
+                focused = e.getSource();
+            }
+
+        };
+        resultSetTable.addFocusListener(focusListener);
+        consoleTextArea.addFocusListener(focusListener);
         // File
         JMenu mFile = createJMenu(getMessage("file"), 'F');
         add(mFile);
-        mFile.add(createJMenuItem(getMessage("file.newwindow"),
-                                  'N',
-                                  new AbstractAction() {
+        mFile.add(createJMenuItem(getMessage("file.newwindow"), 'N', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuNewWindowSelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuNewWindowSelected();
+            }
 
-                                  },
-                                  getKeyStroke(KeyEvent.VK_N, true)));
-        mFile.add(createJMenuItem(getMessage("file.close"),
-                                  'C',
-                                  new AbstractAction() {
+        }, getKeyStroke(KeyEvent.VK_N, true)));
+        mFile.add(createJMenuItem(getMessage("file.close"), 'C', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuCloseSelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuCloseSelected();
+            }
 
-                                  },
-                                  getKeyStroke(KeyEvent.VK_W, true)));
+        }, getKeyStroke(KeyEvent.VK_W, true)));
         mFile.add(new JSeparator());
-        mFile.add(createJMenuItem(getMessage("file.chdir"),
-                                  'D',
-                                  new AbstractAction() {
+        mFile.add(createJMenuItem(getMessage("file.chdir"), 'D', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuChangeDirectorySelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuChangeDirectorySelected();
+            }
 
-                                  },
-                                  getKeyStroke(KeyEvent.VK_D, true)));
-        mFile.add(createJMenuItem(getMessage("file.export"),
-                                  'E',
-                                  new AbstractAction() {
+        }, getKeyStroke(KeyEvent.VK_D, true)));
+        mFile.add(createJMenuItem(getMessage("file.export"), 'E', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuExportSelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuExportSelected();
+            }
 
-                                  }));
+        }));
         mFile.add(new JSeparator());
-        mFile.add(createJMenuItem(getMessage("file.quit"),
-                                  'Q',
-                                  new AbstractAction() {
+        mFile.add(createJMenuItem(getMessage("file.quit"), 'Q', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuQuitSelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuQuitSelected();
+            }
 
-                                  },
-                                  getKeyStroke(KeyEvent.VK_Q, true)));
+        }, getKeyStroke(KeyEvent.VK_Q, true)));
         // Edit
         JMenu mEdit = createJMenu(getMessage("edit"), 'E');
         add(mEdit);
-        mEdit.add(createJMenuItem(getMessage("edit.cut"),
-                                  'T',
-                                  new DefaultEditorKit.CutAction(),
-                                  getKeyStroke(KeyEvent.VK_X, true)));
-        mEdit.add(createJMenuItem(getMessage("edit.copy"),
-                                  'C',
-                                  new DefaultEditorKit.CopyAction(),
-                                  getKeyStroke(KeyEvent.VK_C, true)));
-        mEdit.add(createJMenuItem(getMessage("edit.paste"),
-                                  'P',
-                                  new DefaultEditorKit.PasteAction(),
-                                  getKeyStroke(KeyEvent.VK_P, true)));
+        final KeyStroke keyCtrlX = getKeyStroke(KeyEvent.VK_X, true);
+        mEdit.add(createJMenuItem(getMessage("edit.cut"), 'T', new AbstractAction() {
+
+            public void actionPerformed(ActionEvent e) {
+                performKeyEvent(keyCtrlX, e);
+            }
+
+        }, keyCtrlX));
+
+        final KeyStroke keyCtrlC = getKeyStroke(KeyEvent.VK_C, true);
+        mEdit.add(createJMenuItem(getMessage("edit.copy"), 'C', new AbstractAction() {
+
+            public void actionPerformed(ActionEvent e) {
+                performKeyEvent(keyCtrlC, e);
+            }
+
+        }, keyCtrlC));
+        final KeyStroke keyCtrlV = getKeyStroke(KeyEvent.VK_V, true);
+        mEdit.add(createJMenuItem(getMessage("edit.paste"), 'P', new AbstractAction() {
+
+            public void actionPerformed(ActionEvent e) {
+                performKeyEvent(keyCtrlV, e);
+            }
+
+        }, keyCtrlV));
+        final KeyStroke keyCtrlA = getKeyStroke(KeyEvent.VK_A, true);
+        mEdit.add(createJMenuItem(getMessage("edit.selectall"), 'A', new AbstractAction() {
+
+            public void actionPerformed(ActionEvent e) {
+                performKeyEvent(keyCtrlA, e);
+            }
+
+        }, keyCtrlA));
         mEdit.add(new JSeparator());
-        mEdit.add(createJMenuItem(getMessage("edit.gotoedit"),
-                                  'G',
-                                  new AbstractAction() {
+        mEdit.add(createJMenuItem(getMessage("edit.gotoedit"), 'G', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuGoToInputAreaSelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuGoToInputAreaSelected();
+            }
 
-                                  },
-                                  getKeyStroke(KeyEvent.VK_G, true)));
-        mEdit.add(createJMenuItem(getMessage("edit.clear"),
-                                  'M',
-                                  new AbstractAction() {
+        }, getKeyStroke(KeyEvent.VK_G, true)));
+        mEdit.add(createJMenuItem(getMessage("edit.clear"), 'M', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuClearSelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuClearSelected();
+            }
 
-                                  }));
+        }));
         // View
         JMenu mView = createJMenu(getMessage("view"), 'V');
         add(mView);
@@ -160,65 +173,56 @@ final class Menu extends JMenuBar {
                                                            },
                                                            null);
         mView.add(checkBoxShowColumnNumber);
-        mView.add(createJMenuItem(getMessage("view.widen"),
-                                  'W',
-                                  new AbstractAction() {
+        mView.add(createJMenuItem(getMessage("view.widen"), 'W', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuWidenSelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuWidenSelected();
+            }
 
-                                  },
-                                  getKeyStroke(KeyEvent.VK_PERIOD, true)));
-        mView.add(createJMenuItem(getMessage("view.narrow"),
-                                  'N',
-                                  new AbstractAction() {
+        }, getKeyStroke(KeyEvent.VK_PERIOD, true)));
+        mView.add(createJMenuItem(getMessage("view.narrow"), 'N', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuNarrowSelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuNarrowSelected();
+            }
 
-                                  },
-                                  getKeyStroke(KeyEvent.VK_COMMA, true)));
-        mView.add(createJMenuItem(getMessage("view.adjustcolumnwidth"),
-                                  'A',
-                                  new AbstractAction() {
+        }, getKeyStroke(KeyEvent.VK_COMMA, true)));
+        mView.add(createJMenuItem(getMessage("view.adjustcolumnwidth"), 'A', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuAdjustColumnWidthSelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuAdjustColumnWidthSelected();
+            }
 
-                                  },
-                                  getKeyStroke(KeyEvent.VK_SLASH, true)));
-        JMenu viewColumnAdjustMode = createJMenu(getMessage("view.columnautoadjustmentmode"),
-                                                 'M');
+        }, getKeyStroke(KeyEvent.VK_SLASH, true)));
+        JMenu viewColumnAdjustMode = createJMenu(getMessage("view.columnautoadjustmentmode"), 'M');
         mView.add(viewColumnAdjustMode);
-        columnAdjustModeButtonGroup = new ColumnAdjustModeButtonGroup(this,
-                                                                      viewColumnAdjustMode);
+        columnAdjustModeButtonGroup = new ColumnAdjustModeButtonGroup(this, viewColumnAdjustMode);
         mView.add(new JSeparator());
+        mView.add(createJMenuItem(getMessage("view.refresh"), 'R', new AbstractAction() {
+
+            public void actionPerformed(ActionEvent e) {
+                op.refresh();
+            }
+
+        }, getKeyStroke(KeyEvent.VK_F5, false)));
         viewLimitCountSpinner = new JSpinner(new SpinnerNumberModel(DEFAULT_LIMIT_COUNT,
                                                                     1,
                                                                     Integer.MAX_VALUE,
                                                                     500));
-        mView.add(createJMenuItem(getMessage("view.limitcount"),
-                                  'L',
-                                  new AbstractAction() {
+        mView.add(createJMenuItem(getMessage("view.limitcount"), 'L', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuSetLimitCountSelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuSetLimitCountSelected();
+            }
 
-                                  }));
-        mView.add(createJMenuItem(getMessage("view.font"),
-                                  'F',
-                                  new AbstractAction() {
+        }));
+        mView.add(createJMenuItem(getMessage("view.font"), 'F', new AbstractAction() {
 
-                                      public void actionPerformed(ActionEvent e) {
-                                          onMenuChangeFontSelected();
-                                      }
+            public void actionPerformed(ActionEvent e) {
+                onMenuChangeFontSelected();
+            }
 
-                                  },
-                                  getKeyStroke(KeyEvent.VK_SEMICOLON, true)));
+        }, getKeyStroke(KeyEvent.VK_SEMICOLON, true)));
         // Connect
         JMenu mConnect = createJMenu(getMessage("connect"), 'C');
         add(mConnect);
@@ -226,15 +230,13 @@ final class Menu extends JMenuBar {
                                      'C',
                                      new ConnectorListAction(op, env),
                                      getKeyStroke(KeyEvent.VK_SPACE, true)));
-        mConnect.add(createJMenuItem(getMessage("connect.disconnect"),
-                                     'D',
-                                     new AbstractAction() {
+        mConnect.add(createJMenuItem(getMessage("connect.disconnect"), 'D', new AbstractAction() {
 
-                                         public void actionPerformed(ActionEvent e) {
-                                             onMenuDisconnectSelected();
-                                         }
+            public void actionPerformed(ActionEvent e) {
+                onMenuDisconnectSelected();
+            }
 
-                                     }));
+        }));
         mConnect.add(new JSeparator());
         mConnect.add(createJMenuItem(getMessage("connect.encryptionkey"),
                                      'K',
@@ -257,52 +259,66 @@ final class Menu extends JMenuBar {
         // Command
         JMenu mCmd = createJMenu(getMessage("command"), 'M');
         add(mCmd);
-        mCmd.add(createJMenuItem(getMessage("command.execute"),
-                                 'X',
-                                 new AbstractAction() {
+        mCmd.add(createJMenuItem(getMessage("command.execute"), 'X', new AbstractAction() {
 
-                                     public void actionPerformed(ActionEvent e) {
-                                         onMenuCommandExecuteSelected();
-                                     }
+            public void actionPerformed(ActionEvent e) {
+                onMenuCommandExecuteSelected();
+            }
 
-                                 },
-                                 getKeyStroke(KeyEvent.VK_M, true)));
+        }, getKeyStroke(KeyEvent.VK_M, true)));
         mCmd.add(new JSeparator());
-        mCmd.add(createJMenuItem(getMessage("command.history.previous"),
-                                 'P',
-                                 new AbstractAction() {
+        mCmd.add(createJMenuItem(getMessage("command.history.previous"), 'P', new AbstractAction() {
 
-                                     public void actionPerformed(ActionEvent e) {
-                                         onMenuCommandPreviousHistorySelected();
-                                     }
+            public void actionPerformed(ActionEvent e) {
+                onMenuCommandPreviousHistorySelected();
+            }
 
-                                 },
-                                 getKeyStroke(KeyEvent.VK_K, true)));
-        mCmd.add(createJMenuItem(getMessage("command.history.next"),
-                                 'N',
-                                 new AbstractAction() {
+        }, getKeyStroke(KeyEvent.VK_K, true)));
+        mCmd.add(createJMenuItem(getMessage("command.history.next"), 'N', new AbstractAction() {
 
-                                     public void actionPerformed(ActionEvent e) {
-                                         onMenuCommandNextHistorySelected();
-                                     }
+            public void actionPerformed(ActionEvent e) {
+                onMenuCommandNextHistorySelected();
+            }
 
-                                 },
-                                 getKeyStroke(KeyEvent.VK_J, true)));
+        }, getKeyStroke(KeyEvent.VK_J, true)));
         // Help
         JMenu mHelp = createJMenu(getMessage("help"), 'H');
         add(mHelp);
-        mHelp.add(createJMenuItem(getMessage("help.showhelp"),
-                                  'H',
-                                  new HelpInfoAction(window)));
+        mHelp.add(createJMenuItem(getMessage("help.showhelp"), 'H', new HelpInfoAction(window)));
         mHelp.add(new JSeparator());
-        mHelp.add(createJMenuItem(getMessage("help.about"),
-                                  'A',
-                                  new VersionInfoAction(window)));
+        mHelp.add(createJMenuItem(getMessage("help.about"), 'A', new VersionInfoAction(window)));
         // end
+        bindCancelEditingEvent();
         loadConfiguration();
         validate();
     }
 
+    /**
+     * ҏW~̃Cxg蓖ĂB
+     * ύX#0211̑ΉB
+     */
+    private void bindCancelEditingEvent() {
+        final JTable resultSetTable = this.resultSetTable;
+        ActionListener a = new ActionListener() {
+
+            public void actionPerformed(ActionEvent e) {
+                resultSetTable.editingCanceled(new ChangeEvent(e.getSource()));
+            }
+
+        };
+        final int menuCount = getMenuCount();
+        for (int menuIndex = 0; menuIndex < menuCount; menuIndex++) {
+            JMenu m = getMenu(menuIndex);
+            final int itemCount = m.getItemCount();
+            for (int itemIndex = 0; itemIndex < itemCount; itemIndex++) {
+                JMenuItem item = m.getItem(itemIndex);
+                if (item != null && item.getAccelerator() != null) {
+                    item.addActionListener(a);
+                }
+            }
+        }
+    }
+
     /**
      * ݒ̕B
      */
@@ -344,9 +360,7 @@ final class Menu extends JMenuBar {
      * @param action ANV
      * @return JMenuItem
      */
-    private static JMenuItem createJMenuItem(String caption,
-                                             char mnemonic,
-                                             Action action) {
+    private static JMenuItem createJMenuItem(String caption, char mnemonic, Action action) {
         return createJMenuItem(caption, mnemonic, action, null);
     }
 
@@ -404,9 +418,7 @@ final class Menu extends JMenuBar {
      * @return KeyStroke
      */
     private static KeyStroke getKeyStroke(int key, boolean withControlKey) {
-        return KeyStroke.getKeyStroke(key, (withControlKey
-                ? InputEvent.CTRL_MASK
-                : 0));
+        return KeyStroke.getKeyStroke(key, (withControlKey ? InputEvent.CTRL_MASK : 0));
     }
 
     /**
@@ -417,6 +429,25 @@ final class Menu extends JMenuBar {
         op.requestCommand(command, "menu:" + command);
     }
 
+    /**
+     * Cxg̏B
+     * @param keyStroke L[
+     * @param event Cxg 
+     */
+    void performKeyEvent(KeyStroke keyStroke, ActionEvent event) {
+        JComponent target;
+        if (focused == resultSetTable) {
+            target = resultSetTable;
+        } else {
+            target = consoleTextArea;
+        }
+        ActionListener action = target.getActionForKeyStroke(keyStroke);
+        if (action != null) {
+            event.setSource(target);
+            action.actionPerformed(event);
+        }
+    }
+
     /**
      * j["VEBhE"ȈB
      */
@@ -467,8 +498,7 @@ final class Menu extends JMenuBar {
             }
             try {
                 resultSetTable.export(file);
-                DialogMessage.notice(window,
-                                     getMessage("message.notice.exported"));
+                DialogMessage.notice(window, getMessage("message.notice.exported"));
             } catch (IOException ex) {
                 op.onError(ex);
             }
@@ -622,8 +652,7 @@ final class Menu extends JMenuBar {
      */
     void onMenuConnectorConfigurationSelected() {
         if (env.getCurrentConnector() != null) {
-            DialogMessage.notice(window,
-                                 getMessage("message.notice.starteditconnector"));
+            DialogMessage.notice(window, getMessage("message.notice.starteditconnector"));
         }
         op.requestShowConnectorMapEditDialog();
         env.loadConnectorMap();
@@ -707,8 +736,7 @@ final class Menu extends JMenuBar {
             this.menu = menu;
             String prefix = "view.columnautoadjustmentmode";
             JRadioButtonMenuItem[] radios = {
-                                             createJRadioButtonMenuItem(getMessage(prefix
-                                                                                   + ".none"),
+                                             createJRadioButtonMenuItem(getMessage(prefix + ".none"),
                                                                         false,
                                                                         'N'),
                                              createJRadioButtonMenuItem(getMessage(prefix
