@@ -24,12 +24,12 @@ class ResultSetTable extends JTable {
 
     private final Environment env;
     private final ResultSetTableHeaderCellRenderer headerRenderer;
+    private final ResultSetTableRowHeader rowHeader;
 
     private int limitCount;
     private boolean[] sortedAscArray;
     private String columnAdjustMode;
     private int dragStartColumn;
-    private int dragStartRow;
 
     /**
      * ResultSetTable̐B
@@ -38,6 +38,7 @@ class ResultSetTable extends JTable {
     ResultSetTable(Environment env) {
         this.env = env;
         this.headerRenderer = new ResultSetTableHeaderCellRenderer(getTableHeader().getDefaultRenderer());
+        this.rowHeader = new ResultSetTableRowHeader(this);
         this.limitCount = Integer.MAX_VALUE;
         this.sortedAscArray = new boolean[0];
         initialize();
@@ -79,31 +80,13 @@ class ResultSetTable extends JTable {
      * B
      */
     private void initialize() {
+        ResultSetTableAction.bindAction(this);
         getTableHeader().setDefaultRenderer(headerRenderer);
-        addMouseListener(new ContextMenu(this));
-        addMouseListener(new MouseAdapter() {
-
-            /* (overridden)
-             * @see java.awt.event.MouseAdapter#mousePressed(java.awt.event.MouseEvent)
-             */
-            public void mousePressed(MouseEvent e) {
-                onTableClicked(e);
-            }
-
-        });
-        addMouseMotionListener(new MouseMotionAdapter() {
-
-            /* (overridden)
-             * @see java.awt.event.MouseMotionAdapter#mouseDragged(java.awt.event.MouseEvent)
-             */
-            public void mouseDragged(MouseEvent e) {
-                onTableClicked(e);
-            }
-
-        });
         JTableHeader header = getTableHeader();
         header.setReorderingAllowed(false);
-        header.addMouseListener(new ContextMenu(header));
+        ContextMenu contextMenu = new ContextMenu(this);
+        addMouseListener(contextMenu);
+        header.addMouseListener(contextMenu);
         header.addMouseListener(new MouseAdapter() {
 
             /* (overridden)
@@ -143,17 +126,22 @@ class ResultSetTable extends JTable {
         }
     }
 
-    /**
-     * 0(ō)̕B
+    /* (overridden)
+     * @see javax.swing.JTable#configureEnclosingScrollPane()
      */
-    void fitColumn0() {
-        TableColumnModel columnModel = getColumnModel();
-        if (columnModel != null && columnModel.getColumnCount() > 0) {
-            getTableHeader().setFont(getFont());
-            TableColumn column0 = columnModel.getColumn(0);
-            JLabel label = new JLabel("  " + (getRowCount() + 10000));
-            label.setFont(getFont());
-            column0.setPreferredWidth((int)label.getPreferredSize().getWidth());
+    protected void configureEnclosingScrollPane() {
+        super.configureEnclosingScrollPane();
+        Container p = getParent();
+        if (p instanceof JViewport) {
+            Container gp = p.getParent();
+            if (gp instanceof JScrollPane) {
+                JScrollPane scrollPane = (JScrollPane)gp;
+                JViewport viewport = scrollPane.getViewport();
+                if (viewport == null || viewport.getView() != this) {
+                    return;
+                }
+                scrollPane.setRowHeaderView(rowHeader);
+            }
         }
     }
 
@@ -204,19 +192,6 @@ class ResultSetTable extends JTable {
         }
     }
 
-    /* (overridden)
-     * @see javax.swing.JTable#selectAll()
-     */
-    public void selectAll() {
-        if (isEditing()) {
-            removeEditor();
-        }
-        if (getRowCount() > 0 && getColumnCount() > 1) {
-            setRowSelectionInterval(0, getRowCount() - 1);
-            setColumnSelectionInterval(1, getColumnCount() - 1);
-        }
-    }
-
     /**
      * TCY{̎擾B
      * @param font ƂȂtHg
@@ -256,7 +231,9 @@ class ResultSetTable extends JTable {
             boolean isMousePressed = id == MouseEvent.MOUSE_PRESSED;
             boolean isMouseDragged = id == MouseEvent.MOUSE_DRAGGED;
             if (isMousePressed || isMouseDragged) {
-                clearSelection();
+                if (!e.isControlDown() && !e.isShiftDown()) {
+                    clearSelection();
+                }
                 int columnIndex = columnAtPoint(p);
                 if (columnIndex < 0 || getColumnCount() <= columnIndex) {
                     return;
@@ -264,9 +241,14 @@ class ResultSetTable extends JTable {
                 int index0;
                 int index1;
                 if (isMousePressed) {
-                    dragStartColumn = columnIndex;
-                    index0 = columnIndex;
-                    index1 = columnIndex;
+                    if (e.isShiftDown()) {
+                        index0 = dragStartColumn;
+                        index1 = columnIndex;
+                    } else {
+                        dragStartColumn = columnIndex;
+                        index0 = columnIndex;
+                        index1 = columnIndex;
+                    }
                 } else if (isMouseDragged) {
                     index0 = dragStartColumn;
                     index1 = columnIndex;
@@ -281,7 +263,7 @@ class ResultSetTable extends JTable {
     /**
      * IB
      * @param columnIndex CfbNX
-     * @deprecated
+     * @deprecated selectColumn(int, int) gpĂB
      */
     void selectColumn(int columnIndex) {
         selectColumn(columnIndex, columnIndex);
@@ -294,45 +276,26 @@ class ResultSetTable extends JTable {
      */
     void selectColumn(int index0, int index1) {
         if (getRowCount() > 0) {
-            clearSelection();
             addColumnSelectionInterval(index0, index1);
             addRowSelectionInterval(getRowCount() - 1, 0);
         }
     }
 
+    /**
+     * 0(ō)̕B
+     * @deprecated sԍ\ ResultSetTableRowHeader ɒu܂B̃\bh͉܂B
+     */
+    void fitColumn0() {
+        // do nothing
+    }
+
     /**
      * e[u}EXNbNꂽ̃CxgB
      * @param e }EXCxg
+     * @deprecated sԍ\ ResultSetTableRowHeader ɒu܂B̃\bh͉܂B
      */
     final void onTableClicked(MouseEvent e) {
-        Point p = new Point(e.getX(), e.getY());
-        if (SwingUtilities.isLeftMouseButton(e)) {
-            int columnIndex = columnAtPoint(p);
-            int id = e.getID();
-            boolean isMousePressed = id == MouseEvent.MOUSE_PRESSED;
-            boolean isMouseDragged = id == MouseEvent.MOUSE_DRAGGED;
-            if (columnIndex == 0 && (isMousePressed || isMouseDragged)) {
-                clearSelection();
-                int rowIndex = rowAtPoint(p);
-                if (rowIndex < 0 || getRowCount() < rowIndex) {
-                    return;
-                }
-                int index0;
-                int index1;
-                if (isMousePressed) {
-                    dragStartRow = rowIndex;
-                    index0 = rowIndex;
-                    index1 = rowIndex;
-                } else if (isMouseDragged) {
-                    index0 = dragStartRow;
-                    index1 = rowIndex;
-                } else {
-                    return;
-                }
-                addRowSelectionInterval(index0, index1);
-                addColumnSelectionInterval(getColumnCount() - 1, 1);
-            }
-        }
+        // do nothing
     }
 
     /**
@@ -400,7 +363,7 @@ class ResultSetTable extends JTable {
      */
     void changeColumnWidth(double rate) {
         TableColumnModel columnModel = getColumnModel();
-        for (int i = 1, n = getColumnCount(); i < n; i++) {
+        for (int i = 0, n = getColumnCount(); i < n; i++) {
             TableColumn column = columnModel.getColumn(i);
             int newWidth = (int)(column.getWidth() * rate);
             column.setPreferredWidth(newWidth);
@@ -418,7 +381,7 @@ class ResultSetTable extends JTable {
             // wb_̕ɂ钲
             int fontSize = getFont().getSize();
             TableColumnModel columnModel = getColumnModel();
-            for (int columnIndex = 1, n = getColumnCount(); columnIndex < n; columnIndex++) {
+            for (int columnIndex = 0, n = getColumnCount(); columnIndex < n; columnIndex++) {
                 TableColumn column = columnModel.getColumn(columnIndex);
                 String headerName = String.valueOf(column.getHeaderValue());
                 int size = (headerName.length() + 1) * fontSize;
@@ -433,7 +396,7 @@ class ResultSetTable extends JTable {
             int fontSize = getFont().getSize();
             TableColumnModel columnModel = getColumnModel();
             JTableHeader header = getTableHeader();
-            for (int columnIndex = 1, n = getColumnCount(); columnIndex < n; columnIndex++) {
+            for (int columnIndex = 0, n = getColumnCount(); columnIndex < n; columnIndex++) {
                 TableColumn column = columnModel.getColumn(columnIndex);
                 TableCellRenderer headerRenderer = column.getHeaderRenderer();
                 if (headerRenderer == null) {
@@ -442,8 +405,7 @@ class ResultSetTable extends JTable {
                 int size = 1;
                 if (headerRenderer != null) {
                     size = headerRenderer.getTableCellRendererComponent(this,
-                                                                        getValueAt(0,
-                                                                                   columnIndex),
+                                                                        getValueAt(0, columnIndex),
                                                                         false,
                                                                         false,
                                                                         0,
@@ -451,8 +413,7 @@ class ResultSetTable extends JTable {
                                          .getPreferredSize().width;
                 }
                 for (int rowIndex = 0; rowIndex < rowCount; rowIndex++) {
-                    TableCellRenderer renderer = getCellRenderer(rowIndex,
-                                                                 columnIndex);
+                    TableCellRenderer renderer = getCellRenderer(rowIndex, columnIndex);
                     Object value = getValueAt(rowIndex, columnIndex);
                     Component component = renderer.getTableCellRendererComponent(this,
                                                                                  value,
@@ -460,8 +421,7 @@ class ResultSetTable extends JTable {
                                                                                  false,
                                                                                  rowIndex,
                                                                                  columnIndex);
-                    size = Math.max(size,
-                                    component.getPreferredSize().width / 2);
+                    size = Math.max(size, component.getPreferredSize().width / 2);
                     if (value instanceof java.util.Date) {
                         size *= 2;
                         if (log.isDebugEnabled()) {
@@ -514,11 +474,13 @@ class ResultSetTable extends JTable {
 
         // \̍XV(O)
         final JTableHeader header = this.tableHeader;
+        final DefaultTableModel rowHeaderModel = (DefaultTableModel)rowHeader.getModel();
         EventQueue.invokeLater(new Runnable() {
 
             public void run() {
                 setVisible(false);
                 header.setVisible(false);
+                rowHeaderModel.setRowCount(0);
             }
 
         });
@@ -544,18 +506,15 @@ class ResultSetTable extends JTable {
             }
         }
         ResultSetTableMonitor monitor = new ResultSetTableMonitor();
-        final ResultSetTableModel model = new ResultSetTableModel(cols,
-                                                                  types,
-                                                                  monitor);
+        final ResultSetTableModel model = new ResultSetTableModel(cols, types, monitor, rowHeader);
 
         // [vJn
         int rowCount = 0;
         for (; rs.next(); rowCount++) {
             if (rowCount >= limitCount) {
-                continue;
+                break;
             }
             Vector row = new Vector();
-            row.add(StringClass.EMPTY);
             for (int i = 0; i < cols; i++) {
                 int index;
                 if (needOrderChange) {
@@ -581,11 +540,10 @@ class ResultSetTable extends JTable {
         }
 
         // I
-        sortedAscArray = new boolean[cols + 1];
+        sortedAscArray = new boolean[cols];
         if (!env.getCurrentConnector().isReadOnly()) {
             monitor.prepare(rs, cmd);
         }
-        headerRenderer.setShowUpdatableSign(monitor.isUpdatable());
 
         // \̍XV()
         EventQueue.invokeLater(new Runnable() {
@@ -594,22 +552,32 @@ class ResultSetTable extends JTable {
                 setModel(model);
                 int columnCount = getColumnCount();
                 TableColumnModel columnModel = getColumnModel();
-                TableColumn column0 = columnModel.getColumn(0);
-                column0.setCellRenderer(ResultSetTableRowNumberCellRenderer.INSTANCE);
-                column0.setHeaderValue(StringClass.EMPTY);
-                fitColumn0();
-                column0.setResizable(false);
-                for (int i = 1; i < columnCount; i++) {
+                for (int i = 0; i < columnCount; i++) {
                     TableColumn column = columnModel.getColumn(i);
-                    column.setHeaderValue(headerValues[i - 1]);
+                    column.setHeaderValue(headerValues[i]);
                 }
                 adjustColumnWidth();
                 model.addTableModelListener(ResultSetTable.this);
+                setUpdatableSign();
                 validate();
                 header.setVisible(true);
                 setVisible(true);
             }
 
+            /**
+             * XVۃTC̐ݒB
+             */
+            private void setUpdatableSign() {
+                Container p = getParent();
+                if (p != null && p.getParent() instanceof JScrollPane) {
+                    JScrollPane scrollPane = (JScrollPane)p.getParent();
+                    String iconResource = "icon.updatable." + model.isUpdatable() + ".gif";
+                    scrollPane.setCorner(ScrollPaneConstants.UPPER_LEFT_CORNER,
+                                         new JLabel(new ImageIcon(getClass().getResource(iconResource)),
+                                                    SwingConstants.CENTER));
+                }
+            }
+
         });
 
         return rowCount;
