@@ -31,7 +31,7 @@ public final class Environment {
         loadConnectorMap();
         // fBNg
         this.systemDirectory = LocalSystem.getDirectory();
-        this.currentDirectory = new File(".");
+        this.currentDirectory = getInitialCurrentDirectory();
     }
 
     /**
@@ -114,6 +114,20 @@ public final class Environment {
         }
     }
 
+    /**
+     * JgfBNg̎擾B
+     * @return JgfBNg
+     */
+    private File getInitialCurrentDirectory() {
+        if (LocalSystem.containsKey(PropertyKey.DIRECTORY)) {
+            File directory = new File(LocalSystem.getProperty(PropertyKey.DIRECTORY));
+            if (directory.isDirectory()) {
+                return directory;
+            }
+        }
+        return new File(".");
+    }
+
     /**
      * ^CAEgl̏B
      */
